<?php

namespace Crm\Coupon\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Crm\Coupon\Forms\Coupon as CouponForm;
use Crm\Coupon\Http\Requests\CouponResourceRequest;
use Crm\Coupon\Http\Resources\CouponResource;
use Crm\Coupon\Http\Resources\CouponsCollection;
use Crm\Coupon\Models\Coupon;
use Crm\Coupon\Actions\CouponAction;
use Crm\Coupon\Actions\CouponActions;


/**
 * Resource controller class for coupon.
 */
class CouponResourceController extends BaseController
{

    /**
     * Initialize coupon resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = CouponForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('crm.coupon.modules'), 'coupon', guard_url('coupon'));
            return $next($request);
        });
    }

    /**
     * Display a list of coupon.
     *
     * @return Response
     */
    public function index(CouponResourceRequest $request)
    {
        $request = $request->all();
        $page = CouponActions::run('paginate', $request);

        $data = new CouponsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('coupon::coupon.names'))
            ->view('coupon::coupon.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display coupon.
     *
     * @param Request $request
     * @param Model   $coupon
     *
     * @return Response
     */
    public function show(CouponResourceRequest $request, Coupon $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CouponResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('coupon::coupon.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('coupon::coupon.show')
            ->output();
    }

    /**
     * Show the form for creating a new coupon.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CouponResourceRequest $request, Coupon $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CouponResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('coupon::coupon.name'))
            ->view('coupon::coupon.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new coupon.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CouponResourceRequest $request, Coupon $model)
    {
        try {
            $request = $request->all();
            $model = CouponAction::run('store', $model, $request);
            $data = new CouponResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('coupon::coupon.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('coupon/coupon/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/coupon/coupon'))
                ->redirect();
        }

    }

    /**
     * Show coupon for editing.
     *
     * @param Request $request
     * @param Model   $coupon
     *
     * @return Response
     */
    public function edit(CouponResourceRequest $request, Coupon $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CouponResource($model);
        // return view('coupon::coupon.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('coupon::coupon.name'))
            ->view('coupon::coupon.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the coupon.
     *
     * @param Request $request
     * @param Model   $coupon
     *
     * @return Response
     */
    public function update(CouponResourceRequest $request, Coupon $model)
    {
        try {
            $request = $request->all();
            $model = CouponAction::run('update', $model, $request);
            $data = new CouponResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('coupon::coupon.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('coupon/coupon/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('coupon/coupon/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the coupon.
     *
     * @param Model   $coupon
     *
     * @return Response
     */
    public function destroy(CouponResourceRequest $request, Coupon $model)
    {
        try {

            $request = $request->all();
            $model = CouponAction::run('destroy', $model, $request);
            $data = new CouponResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('coupon::coupon.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('coupon/coupon/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('coupon/coupon/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
