<?php

namespace Crm\Coupon\Http\Controllers;

use Exception;
use Crm\Coupon\Actions\CouponWorkflow;
use Crm\Coupon\Http\Requests\CouponWorkflowRequest;
use Crm\Coupon\Http\Resources\CouponResource;
use Crm\Coupon\Models\Coupon;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class CouponWorkflowController extends BaseController
{
    /**
     * Action controller function for coupon.
     *
     * @param Model $coupon
     * @param action next action for the coupon.
     *
     * @return Response
     */
    public function __invoke(CouponWorkflowRequest $request, Coupon $coupon, $trasition)
    {
        try {
            $request = $request->all();
            $coupon = CouponWorkflow::run($trasition, $coupon, $request);
            $data = new CouponResource($coupon);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('coupon::coupon.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('coupon/coupon/' . $coupon->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('coupon/coupon/' . $coupon->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the coupon and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $coupon
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $coupon, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $coupon = Coupon::findBySignedId($coupon);
        $transitions = $this->transitions($coupon, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('coupon::coupon.name') . ' ' . trans('Approvals'))
            ->view('coupon::public.coupon.workflow')
            ->layout('mobile')
            ->data(compact('coupon', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the coupon.
     *
     * @param PublicRequest $request
     * @param string $coupon
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $coupon, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $coupon = Coupon::findBySignedId($coupon);
        $user = user();
        $coupon = CouponWorkflow::run($transition, $coupon, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($coupon, $user)
    {
        $transitions = [];
        foreach ($coupon->workflow()->transitions($coupon) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.coupon.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'coupon' => $coupon->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $coupon->workflow()->form($value);
            $array['label'] = trans('coupon::coupon.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
